package net.sf.nsisant;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.taskdefs.Execute;
import org.apache.tools.ant.taskdefs.LogStreamHandler;
import org.apache.tools.ant.types.Commandline;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.Path;

public class Task extends org.apache.tools.ant.Task {
	protected static final String NSIS_CMD_LINE = "makensis";
	protected static final String NSIS_OPT_VERBOSITY = "V";
	protected static final String NSIS_OPT_OUTPUT = "O";
	protected static final String NSIS_OPT_PAUSE = "PAUSE";
	protected static final String NSIS_OPT_NOCONFIG = "NOCONFIG";
	protected static final String NSIS_OPT_NOCD = "NOCD";
	protected static final String NSIS_OPT_DEFINE = "D";
	protected static final String NSIS_OPT_SCRIPTCMD = "X";
	protected File m_fScript;
	protected Integer m_iVerbosity;
	protected File m_fOut;
	protected Boolean m_bPause;
	protected Boolean m_bNoconfig;
	protected Boolean m_bNocd;
	protected String m_sPath;
	protected char m_cPrefix;
	protected List<Define> m_lDefines;
	protected List<Scriptcmd> m_lScriptcmds;
	protected List<Path> paths;
	protected List<FileSet> fileSets;
	protected String m_sUninstallFileList;

	public Task() {
		this.m_cPrefix = ((java.lang.System.getProperty("os.name")
				.startsWith("Windows")) ? '/' : '-');
		this.m_lDefines = new ArrayList<Define>();
		this.m_lScriptcmds = new ArrayList<Scriptcmd>();
		this.paths = new ArrayList<Path>();
		this.fileSets = new ArrayList<FileSet>();
	}

	public void execute() {
		File uninstallLogFile = new File(this.m_sUninstallFileList);
		if (!uninstallLogFile.getParentFile().exists()) {
			uninstallLogFile = new File(getProject().getBaseDir(),
					this.m_sUninstallFileList);
		}
		try {
			BufferedWriter bufferedWriter = new BufferedWriter(new FileWriter(
					uninstallLogFile));
			for (Iterator<FileSet> localIterator = this.fileSets.iterator(); localIterator
					.hasNext();) {
				DirectoryScanner directoryScanner = localIterator.next()
						.getDirectoryScanner(getProject());
				String[] includedFiles = directoryScanner.getIncludedFiles();
				for (int i = 0; i < includedFiles.length; ++i) {
					bufferedWriter.write(includedFiles[i].replace('/', '\\'));
					bufferedWriter.newLine();
				}
			}
			bufferedWriter.flush();
			bufferedWriter.close();
		} catch (Exception e) {
			throw new BuildException(
					"Cannot generate uninstall file ["
							+ uninstallLogFile.getPath() + "] due to "
							+ e.getMessage(), getLocation());
		}

		if (this.m_fScript == null)
			throw new BuildException("Attribute 'script' is required",
					getLocation());

		if (this.m_sPath == null)
			this.m_sPath = "";
		else if (!(new File(this.m_sPath).exists()))
			throw new BuildException("Specified path does not exist: "
					+ this.m_sPath, getLocation());

		List<String> lArgs = new ArrayList<String>();
		if (this.m_iVerbosity != null)
			lArgs.add(this.m_cPrefix + "V" + this.m_iVerbosity);

		if (this.m_fOut != null)
			lArgs.add(this.m_cPrefix + "O" + this.m_fOut.getPath());

		if (this.m_bPause != null)
			lArgs.add(this.m_cPrefix + "PAUSE");

		if ((this.m_bNoconfig != null) && (this.m_bNoconfig.booleanValue()))
			lArgs.add(this.m_cPrefix + "NOCONFIG");

		if ((this.m_bNocd != null) && (this.m_bNocd.booleanValue()))
			lArgs.add(this.m_cPrefix + "NOCD");

		for (Iterator<Define> it = this.m_lDefines.iterator(); it.hasNext();) {
			Define define = it.next();
			String sName = define.getName();
			if (sName == null)
				throw new BuildException("Attribute 'name' is required",
						getLocation());

			String sValue = define.getValue();
			if (sValue == null)
				throw new BuildException("Attribute 'value' is required",
						getLocation());

			lArgs.add(this.m_cPrefix + "D" + sName + "=" + sValue);
		}

		for (Iterator<Scriptcmd> it = this.m_lScriptcmds.iterator(); it
				.hasNext();) {
			Scriptcmd scriptcmd = it.next();
			String sCmd = scriptcmd.getCmd();
			if (sCmd == null)
				throw new BuildException("Attribute 'cmd' is required",
						getLocation());

			lArgs.add(this.m_cPrefix + "X" + sCmd);
		}

		lArgs.add(this.m_fScript.getPath());

		Commandline cmd = new Commandline();
		cmd.setExecutable(this.m_sPath + "makensis");
		cmd.addArguments((String[]) lArgs.toArray(new String[lArgs.size()]));

		Execute exe = new Execute(new LogStreamHandler(this, 2, 1));
		exe.setAntRun(getProject());
		exe.setWorkingDirectory(getProject().getBaseDir());
		exe.setCommandline(cmd.getCommandline());
		try {
			int i = exe.execute();
			if (i != 0)
				throw new BuildException("Command failed, error code " + i
						+ ": '" + cmd + "'", getLocation());

			log("Successfully compiled script " + this.m_fScript, 2);
		} catch (IOException ex) {
			ex.printStackTrace();
			throw new BuildException("Command failed: '" + cmd + "'", ex,
					getLocation());
		}
	}

	public void addDefine(Define element) {
		this.m_lDefines.add(element);
	}

	public void addScriptcmd(Scriptcmd element) {
		this.m_lScriptcmds.add(element);
	}

	public void setScript(File fScript) {
		this.m_fScript = fScript;
	}

	public void setVerbosity(Verbosity verbosity) {
		try {
			this.m_iVerbosity = new Integer(verbosity.getValue());
		} catch (NumberFormatException ex) {
			ex.printStackTrace();
		}
	}

	public void setOut(File fOut) {
		this.m_fOut = fOut;
	}

	public void setPause(Boolean bPause) {
		this.m_bPause = bPause;
	}

	public void setNoconfig(Boolean bNoconfig) {
		this.m_bNoconfig = bNoconfig;
	}

	public void setNocd(Boolean bNocd) {
		this.m_bNocd = bNocd;
	}

	public void setPrefix(char cPrefix) {
		this.m_cPrefix = cPrefix;
	}

	public void setPath(String path) {
		if (!(path.endsWith("/")))
			path = path + "/";

		this.m_sPath = path;
	}

	public void addFileSet(FileSet fileSet) {
		this.fileSets.add(fileSet);
	}

	public void setUninstallFileList(String path) {
		this.m_sUninstallFileList = path;
	}
}
